@@ -1,4 +1,4 @@
-/* Copyright (C) 1996, 1997, Russell Lang.  All rights reserved.
+/* Copyright (C) 1996-1998, Russell Lang.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -17,7 +17,7 @@
 */
 
 
-// dwmainc.cpp
+// $Id: dwmainc.cpp $
 // Ghostscript DLL loader for Windows 95/NT
 // For WINDOWCOMPAT (console mode) application
 
@@ -28,6 +28,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <dos.h>
+#include <fcntl.h>
+#include <io.h>
 extern "C" {
 #include "gscdefs.h"
 #define GSREVISION gs_revision
@@ -60,6 +62,8 @@ new_main(int argc, char *argv[])
 typedef char FAR * FARARGV_PTR;
 int rc;
 
+    setmode(fileno(stdin), O_BINARY);
+
     // load DLL
     if (gsdll.load(ghInstance, szDllName, GSREVISION)) {
 	char buf[256];
@@ -108,55 +112,81 @@ main(int argc, char *argv[])
 int PASCAL 
 WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
 {
-#if defined(_MSC_VER)    /* MSC doesn't give us _argc and _argv[] so ...   */
 #define MAXCMDTOKENS 128
-	int	_argc=0;
-	LPSTR	_argv[MAXCMDTOKENS];
-	LPSTR	p, q;
-#ifdef __WIN32__
-	_argv[_argc] = "gswin32.exe";
-#else
-	_argv[_argc] = "gswin.exe";
-#endif
-	// Parse command line handling quotes.
-	p = lpszCmdLine;
-	while (*p) {
-	    // for each argument
-	    while ((*p) && (*p == ' '))
-		p++;	// skip over leading spaces
+    // BC++ 4.5 will give us _argc and _argv[], but they will be 
+    // incorrect if there is a space in the program name.
+    // Provide our own parsing code to create argc and argv[]. 
+    int argc;
+    LPSTR argv[MAXCMDTOKENS];
+    LPSTR p;
+    char command[256];
+    char *args;
+    char *d, *e;
+ 
+    if (hPrevInstance) {
+	fputs("Can't run twice", stdout);
+	return FALSE;
+    }
+
+    /* copy the hInstance into a variable so it can be used */
+    ghInstance = hInstance;
+
+    // If called with "gswin32c.exe arg1 arg2"
+    // lpszCmdLine returns:
+    //    "arg1 arg2" if called with CreateProcess(NULL, command, ...)
+    //    "arg2"      if called with CreateProcess(command, args, ...)
+    // GetCommandLine() returns
+    //    ""gswin32c.exe" arg1 arg2" 
+    //          if called with CreateProcess(NULL, command, ...)
+    //    "  arg1 arg2"      
+    //          if called with CreateProcess(command, args, ...)
+    // Consequently we must use GetCommandLine() 
+    p = GetCommandLine();
+
+    argc = 0;
+    args = (char *)malloc(lstrlen(p)+1);
+    if (args == (char *)NULL) {
+	fprintf(stdout, "Insufficient memory in WinMain()\n");
+	return 1;
+    }
+
+   
+    // Parse command line handling quotes.
+    d = args;
+    while (*p) {
+	// for each argument
+
+	if (argc >= MAXCMDTOKENS - 1)
+	    break;
+
+        e = d;
+        while ((*p) && (*p != ' ')) {
 	    if (*p == '\042') {
-	       p++;		// skip "
-	       q = p;
-	       // scan to end of argument
-	       // doesn't handle embedded quotes
-	       while ((*p) && (*p != '\042'))
-		    p++;
-	       _argv[++_argc] = q;
-	       if (*p)
-		    *p++ = '\0';
+		// Remove quotes, skipping over embedded spaces.
+		// Doesn't handle embedded quotes.
+		p++;
+		while ((*p) && (*p != '\042'))
+		    *d++ =*p++;
 	    }
-	    else if (*p) {
-	       // delimited by spaces
-	       q = p;
-	       while ((*p) && (*p != ' '))
-		    p++;
-	       _argv[++_argc] = q;
-	       if (*p)
-		    *p++ = '\0';
-	    }
-	}
-	_argv[++_argc] = (LPSTR)NULL;
-#endif
+	    else 
+		*d++ = *p;
+	    if (*p)
+		p++;
+        }
+	*d++ = '\0';
+	argv[argc++] = e;
 
-	if (hPrevInstance) {
-	    fputs("Can't run twice", stdout);
-	    return FALSE;
-	}
+	while ((*p) && (*p == ' '))
+	    p++;	// Skip over trailing spaces
+    }
+    argv[argc] = NULL;
 
-	/* copy the hInstance into a variable so it can be used */
-	ghInstance = hInstance;
+    if (strlen(argv[0]) == 0) {
+	GetModuleFileName(hInstance, command, sizeof(command)-1);
+	argv[0] = command;
+    }
 
-	return new_main(_argc, _argv);
+    return new_main(argc, argv);
 }
 #endif
 
@@ -197,9 +227,10 @@ char buf[256];
 	    return count;
 	case GSDLL_DEVICE:
 	    if (count) {
-		fputs("mswindll device not supported by the command\n\
-line version of Ghostscript.  Select a different device using\n\
--sDEVICE= as described in use.txt.\n", stdout);
+		fputs("\n\
+The mswindll device is not supported by the command line version of\n\
+Ghostscript.  Select a different device using -sDEVICE= as described\n\
+in Use.htm.\n", stdout);
 	    }
 	    break;
 	case GSDLL_SYNC:
