@@ -1,16 +1,22 @@
-/* Copyright (C) 1989, 1992, 1994 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1992, 1994, 1996 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This file is part of Aladdin Ghostscript.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility to
-  anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU Ghostscript General Public License for full details.
+  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
+  or distributor accepts any responsibility for the consequences of using it,
+  or for whether it serves any particular purpose or works at all, unless he
+  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
+  License (the "License") for full details.
   
+  Every copy of Aladdin Ghostscript must include a copy of the License,
+  normally in a plain ASCII text file named PUBLIC.  The License grants you
+  the right to copy, modify and redistribute Aladdin Ghostscript, but only
+  under certain conditions described in the License.  Among other things, the
+  License requires that the copyright notice and this notice be preserved on
+  all copies.
 */
 
-/* gdevsun.c */
+/*$Id: gdevsun.c $*/
 /* SunView driver */
 #include "gx.h"			/* for gx_bitmap; includes std.h */
 
@@ -107,7 +113,7 @@ static byte reverse_bits[256] = {
 #endif
 
 /* The instance is public. */
-gx_device_sun gs_sunview_device = {
+gx_device_sun far_data gs_sunview_device = {
 	std_device_std_body(gx_device_sun, &sun_procs, "sunview",
 	  (int)(PAPER_X*DEFAULT_DPI), (int)(PAPER_Y*DEFAULT_DPI),	/* x and y extent */
 	  DEFAULT_DPI, DEFAULT_DPI	/* x and y density */
