@@ -1,27 +1,22 @@
-/* Copyright (C) 1993 Aladdin Enterprises.  All rights reserved.
-  
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility to
-  anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer to
-  the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given to
-  you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
-  
-  Aladdin Enterprises is not affiliated with the Free Software Foundation or
-  the GNU Project.  GNU Ghostscript, as distributed by Aladdin Enterprises,
-  does not depend on any other GNU software.
-*/
+/* Copyright (C) 1993, 1997, 1998 Aladdin Enterprises.  All rights reserved.
 
-/* gdevxxf.c */
+   This file is part of Aladdin Ghostscript.
+
+   Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
+   or distributor accepts any responsibility for the consequences of using it,
+   or for whether it serves any particular purpose or works at all, unless he
+   or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
+   License (the "License") for full details.
+
+   Every copy of Aladdin Ghostscript must include a copy of the License,
+   normally in a plain ASCII text file named PUBLIC.  The License grants you
+   the right to copy, modify and redistribute Aladdin Ghostscript, but only
+   under certain conditions described in the License.  Among other things, the
+   License requires that the copyright notice and this notice be preserved on
+   all copies.
+ */
+
+/*$Id: gdevxxf.c $ */
 /* External font (xfont) implementation for X11. */
 #include "math_.h"
 #include "memory_.h"
@@ -50,17 +45,18 @@ private xfont_proc_char_xglyph(x_char_xglyph);
 private xfont_proc_char_metrics(x_char_metrics);
 private xfont_proc_render_char(x_render_char);
 private xfont_proc_release(x_release);
-private gx_xfont_procs x_xfont_procs = {
-	x_lookup_font,
-	x_char_xglyph,
-	x_char_metrics,
-	x_render_char,
-	x_release
+private const gx_xfont_procs x_xfont_procs =
+{
+    x_lookup_font,
+    x_char_xglyph,
+    x_char_metrics,
+    x_render_char,
+    x_release
 };
 
 /* Return the xfont procedure record. */
-gx_xfont_procs *
-x_get_xfont_procs(gx_device *dev)
+const gx_xfont_procs *
+x_get_xfont_procs(gx_device * dev)
 {
     return &x_xfont_procs;
 }
@@ -68,21 +64,22 @@ x_get_xfont_procs(gx_device *dev)
 /* Define a X11 xfont. */
 typedef struct x_xfont_s x_xfont;
 struct x_xfont_s {
-      gx_xfont_common common;
-      gx_device_X *xdev;
-      XFontStruct *font;
-      int encoding_index;
-      int My;
-      int angle;
+    gx_xfont_common common;
+    gx_device_X *xdev;
+    XFontStruct *font;
+    int encoding_index;
+    int My;
+    int angle;
 };
+
 gs_private_st_dev_ptrs1(st_x_xfont, x_xfont, "x_xfont",
-  x_xfont_enum_ptrs, x_xfont_reloc_ptrs, xdev);
+			x_xfont_enum_ptrs, x_xfont_reloc_ptrs, xdev);
 
 /* Look up a font. */
 private gx_xfont *
-x_lookup_font(gx_device *dev, const byte *fname, uint len,
-	      int encoding_index, const gs_uid *puid, const gs_matrix *pmat,
-	      gs_memory_t *mem)
+x_lookup_font(gx_device * dev, const byte * fname, uint len,
+	    int encoding_index, const gs_uid * puid, const gs_matrix * pmat,
+	      gs_memory_t * mem)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
     x_xfont *xxf;
@@ -97,21 +94,26 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
     Boolean My;
     Boolean scalable_font = False;
 
-    if (!xdev->useXFonts) return NULL;
+    if (!xdev->useXFonts)
+	return NULL;
 
     if (pmat->xy == 0 && pmat->yx == 0) {
-	xwidth  = fabs(pmat->xx * 1000) + 0.5;
+	xwidth = fabs(pmat->xx * 1000) + 0.5;
 	xheight = fabs(pmat->yy * 1000) + 0.5;
-	height  = fabs(pmat->yy * 1000);
-	if (pmat->xx > 0) angle = 0;
-	else angle = 180;
+	height = fabs(pmat->yy * 1000);
+	if (pmat->xx > 0)
+	    angle = 0;
+	else
+	    angle = 180;
 	My = (pmat->xx > 0 && pmat->yy > 0) || (pmat->xx < 0 && pmat->yy < 0);
     } else if (pmat->xx == 0 && pmat->yy == 0) {
-	xwidth  = fabs(pmat->xy * 1000) + 0.5;
+	xwidth = fabs(pmat->xy * 1000) + 0.5;
 	xheight = fabs(pmat->yx * 1000) + 0.5;
-	height  = fabs(pmat->yx * 1000);
-	if (pmat->yx < 0) angle = 90;
-	else angle = 270;
+	height = fabs(pmat->yx * 1000);
+	if (pmat->yx < 0)
+	    angle = 90;
+	else
+	    angle = 270;
 	My = (pmat->yx > 0 && pmat->xy < 0) || (pmat->yx < 0 && pmat->xy > 0);
     } else {
 	return NULL;
@@ -122,12 +124,13 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
     /* or very large fonts, where we can do just as good a job and */
     /* the server may lock up the entire window system while rasterizing */
     /* the whole font. */
-    if ( xwidth < min_X_font_size || xwidth > max_X_font_size ||
-	 xheight < min_X_font_size || xheight > max_X_font_size
-       )
-      return NULL;
+    if (xwidth < min_X_font_size || xwidth > max_X_font_size ||
+	xheight < min_X_font_size || xheight > max_X_font_size
+	)
+	return NULL;
 
-    if (!xdev->useFontExtensions && (My || angle != 0)) return NULL;
+    if (!xdev->useFontExtensions && (My || angle != 0))
+	return NULL;
 
     if (encoding_index == 0 || encoding_index == 1) {
 	int tried_other_encoding = 0;
@@ -135,10 +138,12 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 	fmp = xdev->regular_fonts;
 	while (fmp) {
 	    if (len == strlen(fmp->ps_name) &&
-		strncmp(fmp->ps_name, (const char *)fname, len) == 0) break;
+		strncmp(fmp->ps_name, (const char *)fname, len) == 0)
+		break;
 	    fmp = fmp->next;
 	}
-	if (fmp == NULL) return NULL;
+	if (fmp == NULL)
+	    return NULL;
 	while (True) {
 	    if (encoding_index == 0) {
 		if (fmp->std_count == -1) {
@@ -149,7 +154,7 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 		}
 		if (fmp->std_count) {
 		    for (i = 0; i < fmp->std_count; i++) {
-			char *szp = fmp->std_names[i] + strlen(fmp->x11_name)+1;
+			char *szp = fmp->std_names[i] + strlen(fmp->x11_name) + 1;
 
 			size = 0;
 			while (*szp >= '0' && *szp <= '9')
@@ -169,9 +174,11 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 				xheight, "-0-0-0-*-0-Adobe-fontspecific");
 			x11fontname = x11template;
 		    }
-		    if (x11fontname) break;
+		    if (x11fontname)
+			break;
 		}
-		if (tried_other_encoding) return NULL;
+		if (tried_other_encoding)
+		    return NULL;
 		encoding_index = 1;
 		tried_other_encoding = 1;
 	    } else if (encoding_index == 1) {
@@ -183,7 +190,7 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 		}
 		if (fmp->iso_count) {
 		    for (i = 0; i < fmp->iso_count; i++) {
-			char *szp = fmp->iso_names[i] + strlen(fmp->x11_name)+1;
+			char *szp = fmp->iso_names[i] + strlen(fmp->x11_name) + 1;
 
 			size = 0;
 			while (*szp >= '0' && *szp <= '9')
@@ -203,23 +210,28 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 				xheight, "-0-0-0-*-0-ISO8859-1");
 			x11fontname = x11template;
 		    }
-		    if (x11fontname) break;
+		    if (x11fontname)
+			break;
 		}
-		if (tried_other_encoding) return NULL;
+		if (tried_other_encoding)
+		    return NULL;
 		encoding_index = 0;
 		tried_other_encoding = 1;
 	    }
 	}
     } else if (encoding_index == 2 || encoding_index == 3) {
-	if (encoding_index == 2) fmp = xdev->symbol_fonts;
-	if (encoding_index == 3) fmp = xdev->dingbat_fonts;
+	if (encoding_index == 2)
+	    fmp = xdev->symbol_fonts;
+	if (encoding_index == 3)
+	    fmp = xdev->dingbat_fonts;
 	while (fmp) {
 	    if (len == strlen(fmp->ps_name) &&
 		strncmp(fmp->ps_name, (const char *)fname, len) == 0)
 		break;
 	    fmp = fmp->next;
 	}
-	if (fmp == NULL) return NULL;
+	if (fmp == NULL)
+	    return NULL;
 	if (fmp->std_count == -1) {
 	    sprintf(x11template, "%s%s", fmp->x11_name,
 		    "-*-*-*-*-*-*-Adobe-fontspecific");
@@ -228,7 +240,7 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 	}
 	if (fmp->std_count) {
 	    for (i = 0; i < fmp->std_count; i++) {
-		char *szp = fmp->std_names[i] + strlen(fmp->x11_name)+1;
+		char *szp = fmp->std_names[i] + strlen(fmp->x11_name) + 1;
 
 		size = 0;
 		while (*szp >= '0' && *szp <= '9')
@@ -247,7 +259,8 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 			"-0-0-0-*-0-Adobe-fontspecific");
 		x11fontname = x11template;
 	    }
-	    if (!x11fontname) return NULL;
+	    if (!x11fontname)
+		return NULL;
 	} else {
 	    return NULL;
 	}
@@ -256,15 +269,15 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
     }
 
     if (xwidth != xheight || angle != 0 || My) {
-	if (!xdev->useScalableFonts || !scalable_font) return NULL;
+	if (!xdev->useScalableFonts || !scalable_font)
+	    return NULL;
 	sprintf(x11template, "%s%s+%d-%d+%d%s",
 		fmp->x11_name, My ? "+My" : "",
-		angle*64 , xheight, xwidth,
+		angle * 64, xheight, xwidth,
 		encoding_index == 1 ? "-0-0-0-*-0-ISO8859-1" :
 		"-0-0-0-*-0-Adobe-fontspecific");
 	x11fontname = x11template;
     }
-
     x11font = XLoadQueryFont(xdev->dpy, x11fontname);
     if (x11font == NULL)
 	return NULL;
@@ -292,7 +305,7 @@ x_lookup_font(gx_device *dev, const byte *fname, uint len,
 
 /* Convert a character name or index to an xglyph code. */
 private gx_xglyph
-x_char_xglyph(gx_xfont *xf, gs_char chr, int encoding_index,
+x_char_xglyph(gx_xfont * xf, gs_char chr, int encoding_index,
 	      gs_glyph glyph, gs_proc_glyph_name_t glyph_name_proc)
 {
     x_xfont *xxf = (x_xfont *) xf;
@@ -306,7 +319,8 @@ x_char_xglyph(gx_xfont *xf, gs_char chr, int encoding_index,
 	    chr = gs_map_iso_to_std[chr];
 	else
 	    return gx_no_xglyph;
-	if (chr == 0) return gx_no_xglyph;
+	if (chr == 0)
+	    return gx_no_xglyph;
     }
     if (chr < xxf->font->min_char_or_byte2 ||
 	chr > xxf->font->max_char_or_byte2)
@@ -324,8 +338,8 @@ x_char_xglyph(gx_xfont *xf, gs_char chr, int encoding_index,
 
 /* Get the metrics for a character. */
 private int
-x_char_metrics(gx_xfont *xf, gx_xglyph xg, int wmode,
-	       gs_point *pwidth, gs_int_rect *pbbox)
+x_char_metrics(gx_xfont * xf, gx_xglyph xg, int wmode,
+	       gs_point * pwidth, gs_int_rect * pbbox)
 {
     x_xfont *xxf = (x_xfont *) xf;
 
@@ -375,7 +389,7 @@ x_char_metrics(gx_xfont *xf, gx_xglyph xg, int wmode,
 
 /* Render a character. */
 private int
-x_render_char(gx_xfont *xf, gx_xglyph xg, gx_device *dev,
+x_render_char(gx_xfont * xf, gx_xglyph xg, gx_device * dev,
 	      int xo, int yo, gx_color_index color, int required)
 {
     x_xfont *xxf = (x_xfont *) xf;
@@ -388,12 +402,51 @@ x_render_char(gx_xfont *xf, gx_xglyph xg, gx_device *dev,
 
     if (dev->dname == gs_x11_device.dname) {
 	code = (*xf->common.procs->char_metrics) (xf, xg, 0, &wxy, &bbox);
-	if (code < 0) return code;
-	set_fill_style(FillSolid);
-	set_fore_color(color);
-	set_function(GXcopy);
-	set_font(xxf->font->fid);
-	XDrawString(xdev->dpy, xdev->dest, xdev->gc, xo, yo, &chr, 1);
+	if (code < 0)
+	    return code;
+	/* Buffer text for more efficient X interaction. */
+	if (xdev->text.item_count == MAX_TEXT_ITEMS ||
+	    xdev->text.char_count == MAX_TEXT_CHARS ||
+	    (IN_TEXT(xdev) &&
+	     (yo != xdev->text.origin.y || color != xdev->fore_color ||
+	      xxf->font->fid != xdev->fid))
+	    ) {
+	    DRAW_TEXT(xdev);
+	    xdev->text.item_count = xdev->text.char_count = 0;
+	}
+	if (xdev->text.item_count == 0) {
+	    set_fill_style(FillSolid);
+	    set_fore_color(color);
+	    set_function(GXcopy);
+	    xdev->text.origin.x = xdev->text.x = xo;
+	    xdev->text.origin.y = yo;
+	    xdev->text.items[0].font = xdev->fid = xxf->font->fid;
+	}
+	/*
+	 * The following is wrong for rotated text, but it doesn't matter,
+	 * because the next call of x_render_char will have a different Y.
+	 */
+	{
+	    int index = xdev->text.item_count;
+	    XTextItem *item = &xdev->text.items[index];
+	    char *pchar = &xdev->text.chars[xdev->text.char_count++];
+	    int delta = xo - xdev->text.x;
+
+	    *pchar = chr;
+	    if (index > 0 && delta == 0) {
+		/* Continue the same item. */
+		item[-1].nchars++;
+	    } else {
+		/* Start a new item. */
+		item->chars = pchar;
+		item->nchars = 1;
+		item->delta = delta;
+		if (index > 0)
+		    item->font = None;
+		xdev->text.item_count++;
+	    }
+	    xdev->text.x = xo + wxy.x;
+	}
 	if (xdev->bpixmap != (Pixmap) 0) {
 	    x = xo + bbox.p.x;
 	    y = yo + bbox.p.y;
@@ -404,7 +457,7 @@ x_render_char(gx_xfont *xf, gx_xglyph xg, gx_device *dev,
 	}
 	return 0;
     } else if (!required)
-	return -1;	/* too hard */
+	return -1;		/* too hard */
     else {
 	/* Display on an intermediate bitmap, then copy the bits. */
 	int wbm, raster;
@@ -417,13 +470,15 @@ x_render_char(gx_xfont *xf, gx_xglyph xg, gx_device *dev,
 	dev_proc_copy_mono((*copy_mono)) = dev_proc(dev, copy_mono);
 
 	code = (*xf->common.procs->char_metrics) (xf, xg, 0, &wxy, &bbox);
-	if (code < 0) return code;
+	if (code < 0)
+	    return code;
 	w = bbox.q.x - bbox.p.x;
 	h = bbox.q.y - bbox.p.y;
 	wbm = round_up(w, align_bitmap_mod * 8);
 	raster = wbm >> 3;
 	bits = (byte *) gs_malloc(h, raster, "x_render_char");
-	if (bits == 0) return gs_error_limitcheck;
+	if (bits == 0)
+	    return gs_error_limitcheck;
 	xpm = XCreatePixmap(xdev->dpy, xdev->win, w, h, 1);
 	fgc = XCreateGC(xdev->dpy, xpm, None, NULL);
 	XSetForeground(xdev->dpy, fgc, 0);
@@ -457,11 +512,12 @@ x_render_char(gx_xfont *xf, gx_xglyph xg, gx_device *dev,
 
 /* Release an xfont. */
 private int
-x_release(gx_xfont *xf, gs_memory_t *mem)
+x_release(gx_xfont * xf, gs_memory_t * mem)
 {
 #if 0
     /* The device may not be open.  Cannot reliably free the font. */
     x_xfont *xxf = (x_xfont *) xf;
+
     XFreeFont(xxf->xdev->dpy, xxf->font);
 #endif
     if (mem != NULL)
