@@ -1,38 +1,33 @@
-/* Copyright (C) 1991, 1994, 1996 Aladdin Enterprises.  All rights reserved.
-  
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility to
-  anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer to
-  the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given to
-  you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
-  
-  Aladdin Enterprises is not affiliated with the Free Software Foundation or
-  the GNU Project.  GNU Ghostscript, as distributed by Aladdin Enterprises,
-  does not depend on any other GNU software.
-*/
+/* Copyright (C) 1991, 1994, 1996, 1998 Aladdin Enterprises.  All rights reserved.
 
-/* ostack.h */
+   This file is part of Aladdin Ghostscript.
+
+   Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
+   or distributor accepts any responsibility for the consequences of using it,
+   or for whether it serves any particular purpose or works at all, unless he
+   or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
+   License (the "License") for full details.
+
+   Every copy of Aladdin Ghostscript must include a copy of the License,
+   normally in a plain ASCII text file named PUBLIC.  The License grants you
+   the right to copy, modify and redistribute Aladdin Ghostscript, but only
+   under certain conditions described in the License.  Among other things, the
+   License requires that the copyright notice and this notice be preserved on
+   all copies.
+ */
+
+/*$Id: ostack.h $ */
 /* Definitions for Ghostscript operand stack */
 
 #ifndef ostack_INCLUDED
 #  define ostack_INCLUDED
 
-#include "istack.h"
+#include "iostack.h"
 
 /* Define the operand stack pointers. */
-typedef s_ptr os_ptr;
-typedef const_s_ptr const_os_ptr;
-extern ref_stack o_stack;
+extern op_stack_t iop_stack;
+
+#define o_stack (iop_stack.stack)
 #define osbot (o_stack.bot)
 #define osp (o_stack.p)
 #define ostop (o_stack.top)
@@ -45,18 +40,18 @@ extern ref_stack o_stack;
 /* Operand stack manipulation. */
 
 /* Note that push sets osp to (the new value of) op. */
-/* The do... avoids problems with a possible enclosing 'if'. */
 #define push(n)\
-  do { if ( (op += (n)) > ostop )\
-        { o_stack.requested = (n); return_error(e_stackoverflow); }\
-       else osp = op;\
-  } while (0)
+  BEGIN\
+    if ( (op += (n)) > ostop )\
+      { o_stack.requested = (n); return_error(e_stackoverflow); }\
+    else osp = op;\
+  END
 
 /*
  * Note that the pop macro only decrements osp, not op.  For this reason,
  *
- *	>>>	pop should only be used just before returning,	<<<
- *	>>>	or else op must be decremented explicitly.	<<<
+ *      >>>     pop should only be used just before returning,  <<<
+ *      >>>     or else op must be decremented explicitly.      <<<
  */
 #define pop(n) (osp -= (n))
 
@@ -82,12 +77,12 @@ extern ref_stack o_stack;
  */
 
 /*
- * The operand stack is implemented as a linked list of blocks;
+ * The operand stack is implemented as a linked list of blocks:
  * operators that can push or pop an unbounded number of values, or that
  * access the entire o-stack, must take this into account.  These are:
- *	(int)copy  index  roll  clear  count  cleartomark
- *	counttomark  aload  astore  packedarray
- *	getdeviceprops  putdeviceprops
+ *      (int)copy  index  roll  clear  count  cleartomark
+ *      counttomark  aload  astore  packedarray
+ *      .get/.putdeviceparams .gethardwareparams
  */
 
-#endif					/* ostack_INCLUDED */
+#endif /* ostack_INCLUDED */
